from ..base import BaseExporter
import subprocess

class CustomExporter(BaseExporter):
    def metric_config(self):
        return {
            "xfs_disk_avail_bytes": {"type": "Gauge", "labels": ["filesystem", "user"]},
            "xfs_disk_used_bytes": {"type": "Gauge", "labels": ["filesystem", "user"]},
        }

    def generate(self):
        disk_avail_bytes = self.metrics.get('xfs_disk_avail_bytes')
        disk_used_bytes = self.metrics.get('xfs_disk_used_bytes')
        xfs_data = self._get_xfs_quota(mount='/home')
        for user, usage in xfs_data.items():
            disk_used_bytes.labels('home', user).set(usage['bytes_used'])
            disk_avail_bytes.labels('home', user).set((usage['bytes_total'] - usage['bytes_used']))

    def _get_xfs_quota(self, mount):
        script = "xfs_quota -x -c 'report -N -bi' %s" % mount
        result = subprocess.run(["bash", "-c", script], capture_output=True, text=True)
        data = {}

        for line in result.stdout.strip().splitlines():
            xfs_fields = line.split()
            key = xfs_fields[0]
            if key.startswith("#"):
                # skip users that no longer exist
                continue
            try:
                data[key] = {
                    'bytes_used': float(xfs_fields[1]) * 1000,
                    'bytes_total': float(xfs_fields[3]) * 1000,
                    'inodes_used': float(xfs_fields[6]),
                    'inodes_total': float(xfs_fields[8])
                }
            except Exception:
                # this line is not parseable
                continue
        return data
